(*
    EXMPL02.PAS - Example program for the TGX unit
    A graphical clock
    Copyright (c) 1993-2003,2004 by Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
program Tutor02;
uses Crt, Dos, Tgx;

var
  CX, CY: integer;
  RX, RY: real;

procedure Abort( Msg: string );
begin
  TextMode( CO80 );
  WriteLn( Msg );
  Halt( 1 );
end;

function CircleX( Scale: real; Step, Range: word ): integer;
begin
  CircleX := CX+Round( RX*Scale*Cos(-PI/2+2*PI*Step/Range) );
end;

function CircleY( Scale: real; Step, Range: word ): integer;
begin
  CircleY := CY+Round( RY*Scale*Sin(-PI/2+2*PI*Step/Range) );
end;

(* Draws a line (easiest way I could find...) *)
procedure Line( P: PTgxWindow; X1, Y1, X2, Y2: integer );
var
  I: integer;
begin
  for I:=1 to CX do
    tgxSetPixel( P, X1+I*(X2-X1) div CX, Y1+I*(Y2-Y1) div CX );
end;

var
  P: PTgxWindow;
  I: word;
  H, M, S, C: word;

begin
  (* Initialize the library *)
  TextMode( CO80 );
  if( tgxInit <> 0 ) then
    Abort( 'Cannot initialize TGX library.' );

  (* Create a window *)
  P := tgxCreate( 0, 0,         (* Position *)
                  6, 3,         (* Size *)
                  Yellow,       (* Attribute *)
                  pointer(longint($C0))
                );
  if( P = NIL ) then
    Abort( 'Cannot allocate memory.' );
  tgxClear( P );                (* Clear the window *)
  tgxRefresh( P );              (* Make changes effective *)
  tgxShow( P, 0 );              (* Show window *)

  (* Write the current time *)
  GotoXY( 8, 1 );
  Write( 'A clock in text mode... press any key to exit.' );
  CX := P^.PixelWidth shr 1;    (* Center coords *)
  CY := P^.PixelHeight shr 1;
  RX := CX-2;                   (* Radius *)
  RY := CY-2;
  repeat
    tgxClear( P );
    for I:=0 to 11 do           (* Draw quadrant *)
      tgxSetPixel( P, CircleX( 1, I, 12 ), CircleY( 1, I, 12 ) );
    GetTime( H, M, S, C );
    Line( P, CX, CY, CircleX( 0.9, S, 60 ), CircleY( 0.9, S, 60 ) );
    Line( P, CX, CY, CircleX( 0.8, M, 60 ), CircleY( 0.8, M, 60 ) );
    H := (H mod 12)*30+M shr 1;
    Line( P, CX, CY, CircleX( 0.6, H, 360 ), CircleY( 0.6, H, 360 ) );
    tgxRefresh( P );
  until( KeyPressed );
  while( KeyPressed ) do I := byte(ReadKey);

  (* Destroy window *)
  ClrScr;
  tgxDestroy( P );

  (* Terminate the library and exit to DOS *)
  tgxDone;
end.
